package com.hzu.bookingsystem.controller;

import com.hzu.bookingsystem.VO.ResultVO;
import com.hzu.bookingsystem.converter.Map2Object;
import com.hzu.bookingsystem.dto.UserDTO;
import com.hzu.bookingsystem.bean.UserBean;
import com.hzu.bookingsystem.service.UserService;
import com.hzu.bookingsystem.utils.CookieUtil;
import com.hzu.bookingsystem.utils.MD5Util;
import com.hzu.bookingsystem.utils.ResultVOUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
@RestController
@CrossOrigin
@RequestMapping("/user")
public class UserController {
    @Autowired
    private UserService userService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    // 登录
    @PostMapping("/login")
    public ResultVO login(@RequestBody UserBean user,
                              HttpServletResponse response) {

        //1. openid去和数据库里的数据匹配
        UserBean user1 = userService.findByUnameAndPwd(user.getUsername(),user.getPassword());
        System.out.println(user1);
        if (user1 == null) {
            return ResultVOUtil.error(-2,"账号或密码不正确");
        }

        //2. 设置token至redis
        String token = UUID.randomUUID().toString();
        Integer expire = 7200;
        redisTemplate.opsForValue().set(token, user1.getUId().toString(), expire, TimeUnit.SECONDS);

        //3. 设置token至cookie
        CookieUtil.set(response, "token", token, expire);
        return ResultVOUtil.success();
    }

    // 登出
    @GetMapping("/logout")
    public ResultVO logout(HttpServletRequest request,
                               HttpServletResponse response) {
        //1. 从cookie里查询
        Cookie cookie = CookieUtil.get(request, "token");
        if (cookie != null) {
        //2. 清除redis
            redisTemplate.opsForValue().getOperations().delete(cookie.getValue());
        //3. 清除cookie
            CookieUtil.set(response, "token", null, 0);
        }
        return ResultVOUtil.success();
    }


    // 查看当前用户个人信息
    @GetMapping(value = "/getUserInfo")
    public ResultVO getUserInfo(HttpServletRequest request){
        Integer uId = getUIdByCookie(request);
        UserBean user = userService.findById(uId);
        user.setPassword(null);
        return ResultVOUtil.success(user);
    }

    // 获取当前用户名称
    @GetMapping(value = "/getUserName")
    public ResultVO getUserName(HttpServletRequest request){
        Integer uId = getUIdByCookie(request);
        UserBean user = userService.findById(uId);
        return ResultVOUtil.success(user.getNickname());
    }

    //创建用户
    @PostMapping(value = "/addUser", consumes = "application/json")
    public ResultVO addUser(@RequestBody Map<String,Object> map) {
        // 0.转换对象
        UserBean user = (UserBean) Map2Object.map2Object(map,UserBean.class);
        // 1.查重
        if (userService.findByUserame(user.getUsername()) != null)
            return ResultVOUtil.error(-1,"该工号已存在！");
        // 2.插入
        userService.add(user,(Integer)map.get("position"));
        return ResultVOUtil.success();
    }

    // 通过UID删除用户
    @PostMapping(value = "/deleteUser", consumes = "application/json")
    public ResultVO deleteUser(@RequestBody UserBean user){
        // 查找该用户看是否存在
        UserBean user1 = userService.findById(user.getUId());
        if (user1 == null){
            return ResultVOUtil.error(-1,"用户不存在");
        }
        // 删除该用户
        userService.deleteById(user1.getUId());
        return ResultVOUtil.success();
    }

    //查找用户详情
    @GetMapping(value = "/findById")
    public ResultVO<Map<String, UserBean>> findById(@RequestParam("uId") Integer uId) {
        UserBean user = userService.findById(uId);
        return ResultVOUtil.success(user);
    }

    // 查找所有用户列表
    @GetMapping(value = "/getUserList")
    public ResultVO getUserList(){
        List<UserDTO> userList = userService.findAllUserInfo();
        return ResultVOUtil.success(userList);
    }

    /**
     * 修改自己的用户信息
     * @param user
     * @param request
     * @return
     */
    @PostMapping(value = "/updateBySelf", consumes = "application/json")
    public ResultVO updateBySelf(@RequestBody UserBean user,
                                 HttpServletRequest request){
        Integer uId = getUIdByCookie(request);
        if (uId == null){
            return ResultVOUtil.error(-1,"未找到该用户ID");
        }
        UserBean old = userService.findById(uId);
        user.setUId(uId);
        user.setPassword(old.getPassword());
        user.setUsername(old.getUsername());
        userService.update(user);
        return ResultVOUtil.success();
    }

    /**
     * 修改用户信息
     * @param map
     * @param request
     * @return
     */
    @PostMapping(value = "/updateUser", consumes = "application/json")
    public ResultVO updateUser(@RequestBody Map<String,Object> map,
                                 HttpServletRequest request){
        // 转换对象
        UserBean user = (UserBean) Map2Object.map2Object(map,UserBean.class);
        if (user.getUId() == null){
            return ResultVOUtil.error(-1,"未找到该用户ID");
        }
        UserBean old = userService.findById(user.getUId());
        user.setUId(user.getUId());
        user.setPassword(old.getPassword());
        user.setUsername(old.getUsername());
        userService.update(user,(Integer) map.get("position"));
        return ResultVOUtil.success();
    }

    /**
     * 通过cookie查找Uid
     * @param request
     * @return
     */
    public Integer getUIdByCookie(HttpServletRequest request) {
        Cookie cookie = CookieUtil.get(request, "token");
        if (cookie == null) {
            return null;
        }
        // 从Redis里找该KEY
        String tokenValue = redisTemplate.opsForValue().get(cookie.getValue());
        if (!StringUtils.isEmpty(tokenValue)) {
            // 返回用户UID
            return Integer.valueOf(tokenValue);
        } else
            return null;
    }
}
