package com.hzu.bookingsystem.service.impl;

import com.hzu.bookingsystem.converter.UserBean2UserDTO;
import com.hzu.bookingsystem.dto.UserDTO;
import com.hzu.bookingsystem.bean.UserBean;
import com.hzu.bookingsystem.repository.UserRepository;
import com.hzu.bookingsystem.service.UserAuthService;
import com.hzu.bookingsystem.service.UserService;
import com.hzu.bookingsystem.utils.MD5Util;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.transaction.Transactional;
import java.util.List;
/**
 * author 吴志岳
 */
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private UserAuthService userAuthService;

    @PersistenceContext
    private EntityManager entityManager;

    @Override
    @Transactional
    public UserBean add(UserBean user,Integer groupId) {
        // 1.保证ID为空
        user.setUId(null);
        // 2.加密密码
        user.setPassword(MD5Util.md5Encrypt32Upper(user.getPassword()));
        UserBean userBean = userRepository.save(user);
        userAuthService.add(userBean.getUId(),groupId);
        return userBean;
    }

    @Override
    public List<UserBean> findAll() {
        return userRepository.findAll();
    }

    @Override
    public UserBean findById(Integer uId) {
        return userRepository.findByUId(uId);
    }

    @Override
    public UserBean findByUserame(String username) {
        return userRepository.findByUsername(username);
    }

    @Override
    public UserBean update(UserBean user,Integer groupId) {
        userAuthService.update(user.getUId(),groupId);
        return userRepository.save(user);
    }

    @Override
    public UserBean update(UserBean user) {
        return userRepository.save(user);
    }

    @Override
    @Transactional
    public void deleteById(Integer uId) {
        userRepository.deleteByUId(uId);
    }

    @Override
    public UserBean findByUnameAndPwd(String username, String password) {
        // 加密密码
        password = MD5Util.md5Encrypt32Upper(password);

        return userRepository.findByUsernameAndPassword(username,password);
    }

    @Override
    public List<UserDTO> findAllUserInfo() {
        String sql = "SELECT tb_user.u_id as uId," +
                "tb_user.nickname," +
                "tb_user.tel," +
                "tb_user.email," +
                "tb_user.username," +
                "tb_user.office," +
                "tb_auth_group_access.group_id as position " +
                "FROM " +
                "tb_auth_group_access," +
                "tb_user " +
                "WHERE " +
                "tb_auth_group_access.u_id = tb_user.u_id";
        Query query = entityManager.createNativeQuery(sql);
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(UserDTO.class));
        List<UserDTO> userDTOList = query.getResultList();
        return userDTOList;
    }
}
